import {
  ActionType,
  PageContainer,
  ProForm,
  ProFormText,
  ProFormRadio,
  ProFormDigit,
  ProFormTreeSelect,
  ProTable,
} from '@ant-design/pro-components';
import {Button, Card, Modal} from 'antd';
import React, {useRef} from 'react';
import {AddOutline} from 'antd-mobile-icons';
import {EditBundle, useEdit} from '@/core/hooks/useEdit';
import { deleteMenuRemoveApi, getMenuDetailsApi, postMenuSaveApi, postMenuTreeListApi } from '@/services/swagger/menuAdmin';

const Edit: React.FC<{ bundle: EditBundle;  onSuccess?: () => void }> = (props) => {
  let {bundle} = props;

  const [form] = ProForm.useForm<SectApi.MenuSaveQuery>();
  const [loading, setLoading] = React.useState(false);
  return (
    <Modal
      confirmLoading={loading}
      title={bundle.id ? '编辑菜单' : '添加菜单'}
      open={bundle.open}
      onOk={async () => {
        await form.validateFields();

        try {
          setLoading(true);

          await postMenuSaveApi({
            id: bundle.id,
            ...form.getFieldsValue(),
          });
        } finally {
          setLoading(false);
        }

        props.onSuccess?.();
        bundle.close();
      }}
      onClose={() => {
        form.resetFields();
        bundle.close();
      }}
      onCancel={() => {
        form.resetFields();
        bundle.close();
      }}
    >
      <ProForm
        className={'mt-5'}
        form={form}
        labelCol={{span: 4}}
        layout={'horizontal'}
        submitter={false}
        request={async () => {
          if (bundle.id !== undefined) {
            console.log(bundle.id, 'bundle');
            const res = await getMenuDetailsApi({id: bundle.id});

            return res.data;
          }

          return Promise.resolve({});
        }}

        initialValues={{
          type: 1 ,
          status: 2,
          target: 1,
          hasRefresh: 0
        } }

      >

        <ProFormRadio.Group name='type' label='类型'
                            options={[
                              {label: '目录', value: 1,},
                              {label: '菜单', value: 2},
                              {label: '按钮', value: 3}]}></ProFormRadio.Group>
        <ProFormRadio.Group name='status' label='状态'
                            options={[{label: '正常', value: 1,}, {label: '隐藏', value: 2}]}/>
        <ProFormRadio.Group name='target' label='目标'
                            options={[{label: '页签', value: 1,}, {label: '新窗口', value: 2}]}/>
        <ProFormRadio.Group name='hasRefresh' label='是否刷新'
                            options={[{label: '不刷新', value: 0,}, {label: '刷新', value: 1}]}/>
        <ProFormText name='name' label={'菜单名称'} placeholder={'请输入菜单名称'}
                     rules={[{required: true, message: '请输入菜单名称',}, {
                       min: 3,
                       max: 30,
                       message: '菜单名称长度必须在3-30之间',
                     },]}></ProFormText>

        <ProFormTreeSelect
          label='父菜单' name='parentId'
          style={{width: '100%'}}
          placeholder="请选择父菜单ID"
          allowClear
          request={async () => {
            const transformToTreeData = (data: SectApi.MenuVO[]): any[] =>
              data.map((item) => ({
                title: item.name,
                value: item.id,
                key: item.id,
                children: item.children ? transformToTreeData(item.children) : [],
              }));
            const res = await postMenuTreeListApi({})
              return transformToTreeData(res.data);
          }}
        >
        </ProFormTreeSelect>

        <ProFormText name='icon' label={'图标'} placeholder={'请输入图标'}></ProFormText>

        <ProFormDigit name='sortNum' label={'显示顺序'} placeholder={'请输入显示顺序'}></ProFormDigit>
        <ProFormText name='url' label={'请求地址'} placeholder={'请输入请求地址'}></ProFormText>
        <ProFormText name='perms' label={'权限标识'} placeholder={'请输入权限标识'}></ProFormText>
      </ProForm>
    </Modal>
  );
};


const MenuExp: React.FC = () => {
  const bundle = useEdit();
  const ref = useRef<ActionType>();

  return (
    <>
      <PageContainer content={''}>
        {bundle.open && <Edit bundle={bundle} onSuccess={() => ref.current?.reload()}></Edit>}
        <Card>
          <ProTable<SectApi.MenuVO>
            rowKey={'id'}
            request={async () => {
              const res = await postMenuTreeListApi({})
              return {
                success: res.success,
                data: res.data,
              };
            }}

            columns={[
              {
                title: 'ID',
                dataIndex: 'id',
              },
              {
                title: '菜单名称',
                dataIndex: 'name',
              },
              {
                title: '类型',
                dataIndex: 'type',
                render: (text) => (text === 1 ? '目录' : text === 2 ? '菜单' : '按钮'),
              },
              {
                title: '显示顺序',
                dataIndex: 'sortNum',
              },
              {
                title: '请求地址',
                dataIndex: 'url',
              },
              {
                title: '目标',
                dataIndex: 'target',
                render: (text) => (text === 1 ? '页签' : '新窗口'),
              },
              {
                title: '状态',
                dataIndex: 'status',
                render: (text) => (text === 1 ? '正常' : '隐藏'),
              },
              {
                title: '是否刷新',
                dataIndex: 'hasRefresh',
                render: (text) => (text === 1 ? '刷新' : '不刷新'),
              },
              {
                title: '权限标识',
                dataIndex: 'perms',
              },
              {
                title: '操作',
                valueType: 'option',
                render: (_, record) => [
                  <a key={'edit'} onClick={() => bundle.update(record.id as number)}>
                    编辑
                  </a>,
                  <a
                    key={'delete'}
                    onClick={async () => {
                      if(record.id) {
                        await deleteMenuRemoveApi({id:record.id});
                        ref.current?.reload();
                      }
                    }}
                  >
                    删除
                  </a>,
                ],
              },
            ]}



            pagination={false}
            actionRef={ref}
            toolbar={{
              actions: [
                <Button icon={<AddOutline/>} key={'add'} type={'primary'} onClick={bundle.create}>
                  添加
                </Button>,
              ],
              settings: [],
            }}
            search={false}
          ></ProTable>
        </Card>
      </PageContainer>
    </>
  );
};

export default MenuExp;
