import {ActionType, PageContainer, ProForm, ProFormSelect, ProFormText, ProTable,} from '@ant-design/pro-components';
import {Button, Card, Modal, Dropdown, Input, Space, Tag} from 'antd';
import React, {useRef} from 'react';
import {AddOutline} from 'antd-mobile-icons';
import {EditBundle, useEdit} from '@/core/hooks/useEdit';
import {deleteUserRemoveApi, getUserDetailsApi, postUserPageApi, postUserSaveApi} from '@/services/swagger/userAdmin';
import {getPeakListApi} from "@/services/swagger/peakAdmin";
import {getUserLevelListApi} from "@/services/swagger/userLevelAdmin";
import {getFactionLevelListApi} from "@/services/swagger/factionLevelAdmin";

const Edit: React.FC<{ bundle: EditBundle; onSuccess?: () => void }> = (props) => {
  const {bundle} = props;
  const [form] = ProForm.useForm<SectApi.UserSaveQuery>();
  const [loading, setLoading] = React.useState(false);

  return (
    <Modal
      confirmLoading={loading}
      title={bundle.id ? '编辑用户' : '添加用户'}
      open={bundle.open}
      onOk={async () => {
        await form.validateFields();

        try {
          setLoading(true);

          await postUserSaveApi({
            id: bundle.id,
            ...form.getFieldsValue()
          });
        } finally {
          setLoading(false);
        }

        props.onSuccess?.();
        bundle.close();
      }}
      onCancel={() => {
        form.resetFields();
        bundle.close();
      }}
    >
      <ProForm
        className={'mt-5'}
        form={form}
        labelCol={{span: 4}}
        layout={'horizontal'}
        submitter={false}
        request={async () => {
          if (bundle.id !== undefined) {
            console.log(bundle.id, 'bundle');
            const res = await getUserDetailsApi({id: bundle.id});

            return {
              ...res.data
            }
          }

          return Promise.resolve({});
        }}
      >
        <ProFormText name="name" label={'用户名称'} rules={[{required: true, message: '请输入用户名称',}]}
                     placeholder={'请输入用户名'}></ProFormText>
        <ProFormText name="mobile" label={'手机号'} rules={[{required: true, message: '请输入手机号',}]}
                     placeholder={'请输入手机号'}></ProFormText>
        <ProFormText name="nickname" label={'昵称'} rules={[{required: true, message: '请输入昵称',}]}
                     placeholder={'请输入昵称'}></ProFormText>
        <ProFormText name="purpleCoin" label={'紫币'} rules={[{required: true, message: '请输入紫币',}]}
                     placeholder={'请输入紫币'}></ProFormText>
        <ProFormText name="purpleSpar" label={'紫晶'} rules={[{required: true, message: '请输入紫晶',}]}
                     placeholder={'请输入紫晶'}></ProFormText>

        <ProFormSelect
          name="peakId"
          label="所属峰"
          request={async () => {
            const res = await getPeakListApi();
            const records = res.data ?? []
            return records.map(record => ({
              label: record.name,
              value: record.id,
            }))
          }}
          placeholder="请选择所属峰"
          rules={[{required: true, message: '请选择所属峰'}]}
        />
        <ProFormSelect
          name="userLevel"
          label="会员等级"
          request={async () => {
            const res = await getUserLevelListApi();
            const records = res.data ?? []
            return records.map(record => ({
              label: record.name,
              value: record.id,
            }))
          }}
          placeholder="请选择会员等级"
          rules={[{required: true, message: '请选择会员等级'}]}
        />
        <ProFormSelect
          name="factionLevel"
          label="宗门等级"
          request={async () => {
            const res = await getFactionLevelListApi();
            const records = res.data ?? []
            return records.map(record => ({
              label: record.name,
              value: record.id,
            }))
          }}
          placeholder="请选择宗门等级"
          rules={[{required: true, message: '请选择宗门等级'}]}
        />

        <ProFormText.Password
          name="password"
          label="密码"
          placeholder={'请输入密码'}
          rules={[
            {
              required: true,
              message: '请输入密码',
            },
            {
              min: 6,
              max: 50,
              message: '密码长度必须在6-50之间',
            },
          ]}
        />
      </ProForm>
    </Modal>
  );
};


const Staff: React.FC = () => {
  const bundle = useEdit();
  const ref = useRef<ActionType>();


  return (
    <PageContainer content={''}>
      {bundle.open && <Edit bundle={bundle} onSuccess={() => ref.current?.reload()}></Edit>}
      <Card>
        <ProTable<SectApi.StaffVO>
          rowKey={'id'}

          request={async (params, ...rest) => {
            console.log(params, ...rest)

            const res = await postUserPageApi(
              {
                current: params.current || 1,
                size: params.pageSize || 10,
                ...params
              }
            );

            return {
              success: res.success,
              total: res.data.total,
              data: res.data.records,
            };
          }}
          columns={[
            {
              title: '手机号码',
              dataIndex: 'mobile',
            },
            {
              title: '昵称',
              dataIndex: 'nickname',
            },
            {
              title: '姓名',
              dataIndex: 'name',
            },
            {
              title: '所属峰',
              dataIndex: 'peakName',
              search: false
            },
            {
              title: '所属峰',
              dataIndex: 'peakId',
              valueType: 'select',
              hideInTable: true,
              request: async () => {
                const res = await getPeakListApi();

                return res.data.map(item => ({
                  label: item.name,
                  value: item.id,
                }));
              }
            },
            {
              title: '会员等级',
              dataIndex: 'userLevelName',
              search: false,
            },
            {
              title: '会员等级',
              dataIndex: 'userLevelId',
              valueType: 'select',
              request: async () => {
                const res = await getUserLevelListApi();

                return res.data.map(item => ({
                  label: item.name,
                  value: item.id,
                }));
              }
            },
            {
              title: '宗门等级',
              dataIndex: 'factionLevelName',
              search: false
            },
            {
              title: '宗门等级',
              dataIndex: 'factionLevelId',
              valueType: 'select',
              request: async () => {
                const res = await getFactionLevelListApi();

                return res.data.map(item => ({
                  label: item.name,
                  value: item.id,
                }));
              }
            },
            {
              title: '创建时间',
              dataIndex: 'createTime',
              search: false
            },
            {
              title: '紫币',
              dataIndex: 'purpleCoin',
              search: false
            },
            {
              title: '紫晶',
              dataIndex: 'purpleSpar',
              search: false
            },
            {
              title: '状态',
              dataIndex: 'status',
              valueType: 'select',
              valueEnum: {
                1: '启用',
                0: '禁用'
              }
            },
            {
              title: '创建时间',
              dataIndex: 'createTime',
              valueType: 'dateTimeRange',
              hideInTable: true,
              search: {
                transform: (value) => {
                  return {
                    startTime: value[0],
                    endTime: value[1],
                  };
                },
              }
            },
            {
              title: '紫币',
              dataIndex: 'purpleCoin',
              hideInTable: true,
              valueType: 'digitRange',
              search: {
                transform: (value) => {
                  return {
                    purpleCoinMin: value[0],
                    purpleCoinMax: value[1],
                  };
                },
              }
            },
            {
              title: '紫晶',
              dataIndex: 'purpleSpar',
              hideInTable: true,
              valueType: 'digitRange',
              search: {
                transform: (value) => {
                  return {
                    purpleSparMin: value[0],
                    purpleSparMax: value[1],
                  };
                },
              }
            },

            {
              title: '操作',
              valueType: 'option',
              render: (_, record) => [
                <a key={'edit'} onClick={() => bundle.update(record.id as number)}>
                  编辑
                </a>,
                <a
                  key={'delete'}
                  onClick={async () => {
                    if (record.id) {
                      await deleteUserRemoveApi({id: record.id})
                      ref.current?.reload();
                    }
                  }}
                >
                  删除
                </a>,
              ],
            },
          ]
          }
          search={{
            labelWidth: 'auto',
          }}
          form={{
            ignoreRules: false,
          }}


          actionRef={ref}
          toolbar={{
            actions: [
              <Button icon={<AddOutline/>} key={'add'} type={'primary'} onClick={bundle.create}>
                添加
              </Button>,
            ],
            settings: [],
          }}
        ></ProTable>
      </Card>
    </PageContainer>
  );
};

export default Staff;
