import { isArray, isUndefined } from "lodash"

interface TreeHelperConfig {
  id: string
  children: string
  pid: string
}

type TreeHelperConfigParams = Partial<TreeHelperConfig>

// 默认配置
const DEFAULT_CONFIG: TreeHelperConfig = {
  id: 'id',
  children: 'children',
  pid: 'pid',
}

// 获取配置。  Object.assign 从一个或多个源对象复制到目标对象
const getConfig = (config: TreeHelperConfigParams) => Object.assign({}, DEFAULT_CONFIG, config)

/**
 * @description 对称格式化树
 * @param {T[]} tree
 * @param {R[]} referTree
 * @param {(n: T, r?: R) => Undefinable<T>} formatFunc
 * @param {Partial<DefaultConfig>} config
 * @return {T[]}
 */
export function treeFormatBySymmetry<T = Recordable, R = Recordable>(
  tree: T[],
  referTree: R[],
  formatFunc: (n: T, r?: R) => T | undefined = n => n,
  config: {
    treeConfig?: TreeHelperConfigParams
    referConfig?: TreeHelperConfigParams
  } = {},
): T[] {
  const {
    treeConfig: customTreeConfig = {},
    referConfig: customReferConfig = {},
  } = config

  const treeConfig = getConfig(customTreeConfig)
  const treeChildren = treeConfig.children

  const referConfig = getConfig(customReferConfig)
  const referConfigChildren = referConfig.children

  function treeFormat(tree: T[], referTree?: R[]) {
    const outputTree: T[] = []
    for (let i = 0; i < tree.length; i++) {
      const node: T = tree[i]
      let formatedNode: T | undefined
      let referNode: R | undefined

      if (isArray(referTree) && referTree.length > 0) {
        for (let k = 0; k < referTree.length; k++) {
          const refer = referTree[k]
          formatedNode = formatFunc(node, refer)
          referNode = refer
          if (formatedNode)
            break
        }
      }
      else {
        formatedNode = formatFunc(node)
      }

      if (!isUndefined(formatedNode)) {
        if (isArray((node as Recordable)?.[treeChildren])) {
          (formatedNode as Recordable)[treeChildren] = treeFormat(
            (node as Recordable)[treeChildren],
            (referNode as Recordable)?.[referConfigChildren],
          )
        }

        outputTree.push(formatedNode)
      }
    }

    return outputTree
  }

  return treeFormat(tree, referTree)
}
